# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"read_dicom_from_file": "00_io.ipynb",
         "get_tag_from_loaded_dicom": "00_io.ipynb",
         "dicom_dataframe": "00_io.ipynb",
         "load_mat": "00_io.ipynb",
         "load_h5": "00_io.ipynb",
         "crop": "01_manipulate.ipynb",
         "pad": "01_manipulate.ipynb",
         "resample": "01_manipulate.ipynb",
         "resample_by": "01_manipulate.ipynb",
         "resize": "01_manipulate.ipynb"}

modules = ["io.py",
           "manipulate.py"]

doc_url = "https://UCSF-intelligent-imaging.github.io/I2T2/"

git_url = "https://github.com/UCSF-intelligent-imaging/I2T2/tree/master/"

def custom_doc_links(name): return None
